@@ -1,5 +1,5 @@
 from .server import TelnetServer
 
 __all__ = [
-    'TelnetServer',
+    "TelnetServer",
 ]
